/***************************************************************************
 *   Copyright (C) 2004 by ian reinhart geiser                             *
 *   geiseri@sourcextreme.com                                              *
 *                                                                         * 
 *   Portions of this file are based on KUIViewer in the KDE SDK           *
 *   Copyright (C) 2003,2004 by Richard J. Moore                           *
 *   rich@kde.org                                                          * 
 *   Copyright (C) 2003,2004 by ian reinhart geiser                        *
 *   geiseri@kde.org                                                       *
 *                                                                         * 
 *   This application is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2 of the License, or (at your option) any later version.      *
 *                                                                         *
 *   This application is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this application; if not, write to:                    *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/
 
#include <mainwindow.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qlayout.h>
#include <qdir.h>
#include <qwidgetfactory.h>
#include <qimage.h>
#include <qstringlist.h>
#include <qcombobox.h>
#include <qstylefactory.h> 
#include <qobjectlist.h>
#include <qcursor.h>
#include <qaction.h>
#include <qclipboard.h>
#include <qsettings.h>
#include <qdialog.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qstatusbar.h>
#include <qprinter.h>
#include <qimage.h>

MainWindow::MainWindow ( QWidget * parent, const char *name, WFlags f ) :
	MainWindowBase(parent,name,f)
{
	m_mainWidget = 0L;
	m_currFile = QDir::homeDirPath();
	QHBoxLayout *box = new QHBoxLayout(m_preview, 3, 3, "layoutbox");
	box->setAutoAdd(true);

	QSettings settings;
	settings.setPath("SourceXtreme","QUIViewer");
	settings.beginGroup("Options");
	m_styles->insertStringList( QStyleFactory::keys() );
	m_styles->setCurrentItem(settings.readNumEntry("/style"));
	settings.endGroup();

	updateActions();
	connect( qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));
}

MainWindow::~MainWindow()
{
	if( m_mainWidget )
		delete m_mainWidget;
}

void MainWindow::fileOpen()
{
	QString file = QFileDialog::getOpenFileName ( m_currFile, "User Interface Files (*.ui)", this, "open file", "Please Select File" );

	if( file.isEmpty() )
		return;

	if( m_mainWidget )
	{
		delete m_mainWidget;
		m_mainWidget = 0L;
	}
	openFile( file );
}

void MainWindow::fileExit()
{
	qApp->quit();
}

void MainWindow::helpAbout()
{
	QString aboutMessage = "Copyright 2004 SourceXtreme, Inc. and KDE Project\n";
	aboutMessage += "This application is licensed under the Lesser General Public License\n";
	aboutMessage += "For more information on QUIViewer see http://www.sourcextreme.com/projects/uibuilder";
	QMessageBox::about ( this, "About QUIViewer", aboutMessage );
}

void MainWindow::fileScreenCapture()
{
	if( m_mainWidget )
	{
		QString filter = "Images (";
		QStringList formats;
		QStrList lst = QImageIO::outputFormats();

		QStrList::Iterator idx = lst.begin();
		for( ; idx != lst.end(); ++idx)
		{
			QString currFilter = "*.";
			currFilter += (*idx);
			currFilter += " ";
			filter += currFilter.lower();
		}

		filter += ")";
		
		QString file = QFileDialog::getSaveFileName ( QDir::homeDirPath(), filter, this, "open file", "Save as image..." );
		if( file.isEmpty() )
			return;

		QString ext = file.section('.', -1 ).upper();
		qDebug("Filename ext '%s'", ext.latin1());

		QPixmap pix = QPixmap::grabWidget( m_mainWidget );
		if( !pix.save( file, ext.latin1() ) )
		{
			QString error = QString("The file \"%1\"\n could not be saved.").arg(file);
			QMessageBox::critical ( this, "Error", error );
		}
	}
	else
	{
		// must have active widget
		QMessageBox::critical( this, "Error", "You must first open a UI file before you can save a screenshot of it.");
	}
	
}
void MainWindow::slotHelpAboutQt()
{
	QMessageBox::aboutQt(this);
}

void MainWindow::slotNewStyle( const QString & style )
{
	if( m_mainWidget )
		updateChildren(style);
}

void MainWindow::updateChildren(const QString &style )
{
	QApplication::setOverrideCursor( WaitCursor );
	m_mainWidget->setStyle( style);

	QObjectList *lst = m_mainWidget->queryList( "QWidget" );
	for ( QObject *obj = lst->first(); obj; obj = lst->next() )
		( static_cast<QWidget *>(obj) )->setStyle( style );
	delete lst;

	m_mainWidget->show();

	QSettings settings;
	settings.beginGroup("Options");
	settings.setPath("SourceXtreme","QUIViewer");
	settings.writeEntry("/style", m_styles->currentItem() );
	settings.endGroup();
	QApplication::restoreOverrideCursor();
}
 
void MainWindow::editCopy()
{

    if( m_mainWidget )
    {
	QClipboard *clipboard = QApplication::clipboard();
    	clipboard->setPixmap(QPixmap::grabWidget(m_mainWidget));
    }
    else
	QMessageBox::critical( this, "Error", "You must first open a UI file before you can save a screenshot of it.");

}

void MainWindow::updateActions()
{
	if( m_mainWidget )
	{
		fileScreenCap->setEnabled( true );
		editCopyAction->setEnabled( true );
		filePrintAction->setEnabled( true );
		m_styles->setEnabled( true );
	}
	else
	{
		fileScreenCap->setEnabled( false );
		editCopyAction->setEnabled( false );
		filePrintAction->setEnabled( false );
		m_styles->setEnabled( false );
	}
}

void MainWindow::slotClose()
{
	if( m_mainWidget )
	{
		delete m_mainWidget;
		m_mainWidget = 0L;
	}
	updateActions();
}

void MainWindow::openFile( const QString &file )
{
	m_mainWidget = QWidgetFactory::create ( file, 0L, m_preview, "EmbeddedWidget");

	if( m_mainWidget )
	{
		m_currFile = file;
		updateChildren( m_styles->currentText() );
		QDialog *dlg =  dynamic_cast<QDialog*>(m_mainWidget);
		QMainWindow *win = dynamic_cast<QMainWindow*>(m_mainWidget);
		if( win )
		{
			win->show();
			m_mainWidget->installEventFilter( this );
		}
		else if( dlg )
		{
			updateActions();
			dlg->exec();
			slotClose();
			return;
		}
		else
			m_mainWidget->show();
	}
	else
	{
		QString error = QString("The file \"%1\"\n could not be opened.").arg(file);
		QMessageBox::critical ( this, "Error", error );
	}
	updateActions();
}


void MainWindow::filePrint()
{
	const int Margin = 10;
	QPrinter *printer = new QPrinter(QPrinter::HighResolution);
	QSettings settings;
	settings.setPath("SourceXtreme","QUIViewer");
	settings.beginGroup("Printing");
	printer->setPrinterName( settings.readEntry("/printername") );
	printer->setPageSize((QPrinter::PageSize)settings.readNumEntry("/pagesize") );
	printer->setPaperSource((QPrinter::PaperSource)settings.readNumEntry("/papersource"));
	printer->setOrientation((QPrinter::Orientation)settings.readNumEntry("/orientation"));
	printer->setColorMode((QPrinter::ColorMode)settings.readNumEntry("/colormode"));
	settings.endGroup();
	
	if( printer->setup(this) )
	{
		QApplication::setOverrideCursor( WaitCursor );
		statusBar()->message("Printing the image...");
		QPainter p;
		if( p.begin(printer) )
		{
			QPaintDeviceMetrics metrics( printer );
			int xsize = metrics.width() - Margin*2; 
			int ysize = metrics.height() - Margin*2;
			
			QImage img = QPixmap::grabWidget(m_mainWidget).convertToImage();
			
			p.drawImage(Margin, Margin,img.smoothScale(xsize,ysize,QImage::ScaleMin));
			// Draw the file name below.
			p.end();
		}
		settings.setPath("SourceXtreme","QUIViewer");
		settings.beginGroup("Printing");
		settings.writeEntry("/printername",printer->printerName());
		settings.writeEntry("/pagesize",printer->pageSize());
		settings.writeEntry("/papersource",printer->paperSource());
		settings.writeEntry("/orientation",printer->orientation());
		settings.writeEntry("/colormode",printer->colorMode());
		settings.endGroup();		
		
	}
	delete printer;
	QApplication::restoreOverrideCursor();
	statusBar()->message("Image sent to printer...",2000);
}

bool MainWindow::eventFilter( QObject *obj, QEvent *ev )
{
	if( obj == m_mainWidget )
	{
            if ( ev->type() == QEvent::Close ) 
	    {
		slotClose();		
                return true;
            }
	    else 
	    {
                return false;
            }
	}
	else
		return MainWindowBase::eventFilter(obj,ev);	
}
